#!/usr/bin/env python3

from argparse import ArgumentParser
from re import fullmatch
from sys import argv
from zipfile import ZipFile


def is_included(patterns, name):
    return any([fullmatch(r"{}".format(p), name) for p in patterns])


def zip_read(zip_file, patterns, encoding="utf-8"):  # pragma: no cover
    with ZipFile(zip_file, "r") as z:
        return [
            (name, z.read(name).decode(encoding))
            for name in z.namelist()
            if is_included(patterns, name)
        ]


def main(zip_file, patterns, only_content=False, encoding="utf-8"):
    for file_name, content in zip_read(zip_file, patterns, encoding):
        print("#" * 80)
        if not only_content:
            print("#" * 5, file_name, "#" * 5)
        print(content)


def parse_args(args):
    parser = ArgumentParser(description="Read content from files in zip (jar etc)")
    parser.add_argument(
        "zip_file",
        help="The zip file",
    )
    parser.add_argument(
        "-p",
        "--patterns",
        nargs="+",
        default=[".*"],
        help="File name patterns",
    )
    parser.add_argument(
        "-o",
        "--only_content",
        action="store_true",
        help="Print only content",
    )
    parser.add_argument(
        "-e",
        "--encoding",
        default="utf-8",
        help="File encoding",
    )

    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from difflib import ndiff
from sys import argv
from zipfile import ZipFile, ZIP_DEFLATED


def diff(name, content, updated_content):
    diffs = list(ndiff(content.splitlines(1), updated_content.splitlines(1)))
    if diffs:
        print(f"{name}\n{''.join(diffs)}")


def replace(content, replacements):
    updated_content = content
    for old, new in replacements.items():
        updated_content = updated_content.replace(old, new)
    return updated_content


def zip_read(zip_file, encoding="utf-8"):
    with ZipFile(zip_file, "r") as z:
        for name in z.namelist():
            yield (name, z.read(name).decode(encoding))


def update_zip(
    original_zip, updated_zip, replacements, encoding="utf-8", verbose=False
):
    with ZipFile(updated_zip, "w", compression=ZIP_DEFLATED) as z:
        for name, content in zip_read(original_zip, encoding=encoding):
            updated_content = replace(content, replacements)
            if verbose:
                diff(name, content, updated_content)
            z.writestr(name, updated_content)


def main(original_zip, updated_zip, replacements, encoding="utf-8", verbose=False):
    # make a dict of replacements list
    d = dict(zip(replacements[::2], replacements[1::2]))
    update_zip(original_zip, updated_zip, d, encoding, verbose)


def parse_args(args):
    parser = ArgumentParser(description="Replace in zip (e.g. zip, jar or war")
    parser.add_argument("original_zip", help="original zip path")
    parser.add_argument("updated_zip", help="updated zip path")
    parser.add_argument(
        "replacements",
        nargs="+",
        help="A list of replacements to be done in the zip. Index 0 will be replaced with index 1 and so on.",
    )
    parser.add_argument(
        "-e",
        "--encoding",
        default="utf-8",
        help="TFS projects, e.g organisation/project",
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="Verbose output")

    return parser.parse_args(args)


if __name__ == "__main__":
    """
    Example:
    ./replace_in_zip.py original.zip updated.zip old1 new1 old2 new2 oldN newN
    """
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_repos import get_repos


def get_repo_branches(organisation, project, repository, ref_filter="refs/heads"):
    f = f"/{ref_filter}" if ref_filter else ""
    uri = (
        f"/{organisation}/{project}/_apis/git/repositories"
        f"/{repository}{f}?includeStatuses=true&api-version=3.2"
    )
    return call(uri)["value"]


def get_branches(organisations=None, projects=None, repos=None, names=False):
    if organisations:
        repos = get_repos(organisations=organisations, repo_specs=True)
    if projects:
        repos = get_repos(projects=projects, repo_specs=True)
    if not repos:
        return []

    branch_specs = [(repo, get_repo_branches(*repo.split("/"))) for repo in repos]
    if names:
        return [
            (repo, [branch["name"].split("/")[-1] for branch in branches])
            for repo, branches in branch_specs
        ]
    return [
        (
            repo,
            [dict(branch, name=branch["name"].split("/")[-1]) for branch in branches],
        )
        for repo, branches in branch_specs
    ]


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get TFS repos")
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    required_group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )
    required_group.add_argument(
        "-r", "--repos", nargs="+", help="TFS repos, e.g organisation/project/repo"
    )

    parser.add_argument(
        "-n", "--names", action="store_true", help="Get only repo names"
    )
    return parser.parse_args(args)


def main(organisations, projects, repos, names):
    for the_repo, the_branches in get_branches(
        organisations,
        projects,
        repos,
        names,
    ):
        for the_branch in the_branches:
            print(f"{the_repo}: {the_branch}")


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import datetime
import traceback
from os import makedirs
from shutil import rmtree
from urllib.error import HTTPError

from vang.bitbucket.api import call
from vang.bitbucket.clone_repos import clone, get_repos_commands
from vang.bitbucket.get_repos import get_all_repos
from vang.core.core import pmap_unordered


def to_date(timestamp):
    return datetime.datetime.utcfromtimestamp(timestamp / 1000)


def get_commit_author_dates(repos_specs, branch):
    def f(spec):
        project, repo = spec
        try:
            commits = call(
                f"/rest/api/1.0/projects/{project}"
                f"/repos/{repo}/commits?limit=1&until={branch}"
            )
            author_timestamp = commits["values"][0]["authorTimestamp"]
        except HTTPError:
            # Does not have branch
            author_timestamp = None
        return spec, to_date(author_timestamp) if author_timestamp else None

    return pmap_unordered(f, repos_specs)


def get_repos_with_commits_since(branch, projects, since):
    return [
        spec
        for spec, author_date in get_commit_author_dates(
            get_all_repos(projects, only_spec=True),
            branch,
        )
        if author_date and author_date > since
    ]


def clone_repos(commands, clone_dir):
    rmtree(clone_dir, ignore_errors=True)
    makedirs(clone_dir)
    n = 1
    for process in clone(
        [command for clone_dir, project, repo, command in commands],
        clone_dir,
    ):
        try:
            print(str(n).zfill(2), process.returncode, process.stdout.decode(), end="")
            n += 1
        except OSError:
            print(traceback.format_exc())


def main(since, projects, branch, clone_dir):
    repo_specs = get_repos_with_commits_since(branch, projects, since)
    commands = get_repos_commands([f"{p}/{r}" for p, r in repo_specs], branch, True)
    for c in commands:
        print(c[3])
    clone_repos(commands, clone_dir)


if __name__ == "__main__":  # pragma: no cover
    the_since = datetime.datetime(2018, 1, 1)
    the_projects = ["ESBU", "ESBES"]
    # branch = 'develop'
    # clone_dir = '/Users/ei4577/slask/crap/develop'
    a_branch = "feature/bosse"
    a_clone_dir = "/Users/ei4577/slask/crap/bosse"

    main(the_since, the_projects, a_branch, a_clone_dir)



#!/usr/bin/env python3
import logging
import random
from argparse import ArgumentParser
from random import choice, shuffle
from string import ascii_lowercase, ascii_uppercase, digits
from sys import argv

logging.basicConfig(level=logging.INFO)


def generate_random_password(
    special_characters: str = "!@#$%^&*()",
    number_ascii_lower: int = random.randint(7, 8),
    number_ascii_upper: int = random.randint(3, 4),
    number_digits: int = random.randint(3, 4),
    number_special: int = random.randint(3, 4),
) -> str:
    password = [
        choice(c)
        for c, n in [
            [digits, number_digits],
            [ascii_lowercase, number_ascii_lower],
            [ascii_uppercase, number_ascii_upper],
            [special_characters, number_special],
        ]
        for _ in range(n)
    ]

    shuffle(password)

    logging.info(
        "Password of length %s: %s lowercase letters, %s uppercase letters, %s digits, %s special characters",
        len(password),
        number_ascii_lower,
        number_ascii_upper,
        number_digits,
        number_special,
    )

    return "".join(password)


def parse_args(args):
    parser = ArgumentParser(
        description="Generates a password. "
        "Default 16-20 characters: "
        "lowercase and uppercase letters, digits and special characters."
    )
    parser.add_argument(
        "-s",
        "--special_characters",
        help="Special characters to use in password generation",
    )
    parser.add_argument(
        "-nl",
        "--number_ascii_lower",
        type=int,
        help="Number of ascii lowercase letters in the password",
    )
    parser.add_argument(
        "-nu",
        "--number_ascii_upper",
        type=int,
        help="Number of ascii uppercase letters in the password",
    )
    parser.add_argument(
        "-nd", "--number_digits", type=int, help="Number of digits in the password"
    )
    parser.add_argument(
        "-ns",
        "--number_special",
        type=int,
        help="Number of special characters in the password",
    )
    return parser.parse_args(args)


if __name__ == "__main__":
    the_args = {k: v for (k, v) in parse_args(argv[1:]).__dict__.items() if v}
    print(generate_random_password(**the_args))



#!/usr/bin/env python3

import argparse
import traceback
from itertools import count
from os import makedirs
from sys import argv

from vang.pio.shell import run_commands
from vang.tfs.get_projects import get_projects
from vang.tfs.get_repos import get_repos


def clone(commands, root_dir):
    makedirs(root_dir, exist_ok=True)
    yield from run_commands(
        [(cmd, root_dir) for cmd in commands], max_processes=5, timeout=180, check=False
    )


def get_commands(clone_specs, branch, flat):
    b = f" -b {branch}" if branch else ""
    return [f'git clone {c[0]}{b}{"" if flat else f" {c[1]}"}' for c in clone_specs]


def get_clone_specs(projects, flat):
    return [
        (
            repo[1]["remoteUrl"],
            repo[1]["name"] if flat else f'{repo[0]}/{repo[1]["name"]}',
        )
        for repo in get_repos(projects=projects)
    ]


def clone_repos(
    root_dir, organisations=None, projects=None, repos=None, branch=None, flat=False
):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    elif repos:
        projects = set(["/".join(r.split("/")[:2]) for r in repos])

    clone_specs = get_clone_specs(projects, flat)

    if repos:
        clone_dirs = [r.split("/")[2] if flat else r for r in repos]
        clone_specs = [
            (url, clone_dir)
            for url, clone_dir in clone_specs
            if clone_dir in clone_dirs
        ]

    commands = get_commands(clone_specs, branch, flat)
    for n, process in zip(count(1), clone(commands, root_dir)):
        try:
            print(str(n).zfill(2), process.stdout.decode(), end="")
        except OSError:  # pragma: no cover
            print(traceback.format_exc())

    return clone_specs


def parse_args(args):
    parser = argparse.ArgumentParser(description="Clone TFS repos")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-o", "--organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    group.add_argument(
        "-p", "--projects", nargs="+", help="TFS projects, e.g organisation/project"
    )
    group.add_argument(
        "-r", "--repos", nargs="+", help="Repos, e.g. organisation/project/repo"
    )
    parser.add_argument("-b", "--branch", help="The clone branch")
    parser.add_argument(
        "-d", "--clone_dir", default=".", help="The directory to clone into"
    )
    parser.add_argument(
        "-f", "--flat", help="Clone to flat structure", action="store_true"
    )
    return parser.parse_args(args)


def main(clone_dir, organisations, projects, repos, branch, flat):
    for a_repo in clone_repos(
        clone_dir,
        organisations,
        projects,
        repos,
        branch,
        flat,
    ):
        print(a_repo[1])


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import makedirs
from sys import argv

from requests import get

from vang.artifactory.list_repo_content import get_repo_content
from vang.core.core import pmap_unordered


def download(output_file, url, username, password):
    with open(output_file, "wb") as f:
        f.write(get(url, auth=(username, password)).content)
    return url, output_file


def get_path(base_dir, file):
    return f"{base_dir}{file}"


def get_url(artifactory_url, file):
    return f"{artifactory_url}{file}"


def create_dirs(base_dir, files):
    for f in files:
        if f["folder"]:
            p = get_path(base_dir, f["uri"])
            makedirs(p, exist_ok=True)


def download_files(artifactory_url, base_dir, files, max_processes=10):
    files_and_urls = [
        (get_path(base_dir, f["uri"]), get_url(artifactory_url, f["uri"]))
        for f in files
        if not f["folder"]
    ]
    yield from pmap_unordered(
        lambda uaf: download(uaf[0], uaf[1], "", ""),
        files_and_urls,
        processes=max_processes,
    )


def get_artifactory_url(repo_content):
    return repo_content["uri"].replace("api/storage/", "")


def download_repo_content(output_dir, repo_key):
    base_dir = f"{output_dir}/{repo_key}"
    repo_content = get_repo_content(repo_key)
    create_dirs(base_dir, repo_content["files"])
    artifactory_url = get_artifactory_url(repo_content)
    yield from download_files(artifactory_url, base_dir, repo_content["files"])


def main(output_dir, repo_key):
    for url, output_file in download_repo_content(output_dir, repo_key):
        print(url, "->", output_file)


def parse_args(args):
    parser = ArgumentParser(description="Download repository content")
    parser.add_argument("output_dir", help="Output dir")
    parser.add_argument("repo_key", help="Repository key, e.g. libs-release")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv


def name(d):
    return [
        "monday",
        "tuesday",
        "wednesday",
        "thursday",
        "friday",
        "saturday",
        "sunday",
    ][d - 1]


def zeller(year, month, day):
    """Calculates the weekday.

    :param year: e.g. 2018
    :param month: 1..12
    :param day: 1..31
    :return: the ISO week date Day-of-Week d (1 = Monday to 7 = Sunday)
    """
    y = year - 1 if month < 3 else year
    q = day
    m = month + 12 if month < 3 else month
    k = y % 100
    j = y // 100

    h = (q + 13 * (m + 1) // 5 + k + k // 4 + j // 4 + 5 * j) % 7
    return ((h + 5) % 7) + 1


def parse_args(args):
    parser = ArgumentParser(description="Prints weekday")
    parser.add_argument("year", help="Year")
    parser.add_argument("month", help="Month")
    parser.add_argument("day", help="Day")
    return parser.parse_args(args)


def main(year, month, day):
    print(name(zeller(int(year), int(month), int(day))))


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.misc.s import get_zipped_cases
from vang.pio.rsr import get_replace_function, rsr
from vang.pio.shell import run_command
from vang.tfs.clone_repos import clone_repos
from vang.tfs.create_repo import create_repo


def setup(repo, src_branch, dest_repo, dest_branch, work_dir):
    clone_url, repo_dir = clone_repos(
        work_dir,
        repos=[repo],
        branch=src_branch,
    )[0]

    dest_repo_dir = f"{work_dir}/{dest_repo}"
    run_command(f"mv {repo_dir} {dest_repo_dir}")

    run_command("rm -rf .git", return_output=True, cwd=dest_repo_dir)
    run_command("git init", return_output=True, cwd=dest_repo_dir)
    if not dest_branch == "master":
        run_command(
            f"git checkout -b {dest_branch}",
            return_output=True,
            cwd=dest_repo_dir,
        )
    return clone_url, dest_repo_dir


def commit_all(repo_dir):
    run_command("git add --all", return_output=True, cwd=repo_dir)
    run_command(
        'git commit -m "Initial commit"',
        return_output=True,
        cwd=repo_dir,
    )


def update(replacements, dest_repo_dir):
    for pair in replacements:
        for old, new in set(get_zipped_cases(pair)):
            rsr(
                old,
                new,
                [dest_repo_dir],
                get_replace_function(False),
            )


def create_and_push_to_dest_repo(
    branch,
    dest_repo,
    dest_repo_dir,
):
    dest_repo_origin = create_repo(dest_repo)["remoteUrl"]
    run_command(
        f"git remote add origin {dest_repo_origin}",
        return_output=True,
        cwd=dest_repo_dir,
    )
    run_command(
        f"git push -u origin {branch}",
        return_output=True,
        cwd=dest_repo_dir,
    )
    return dest_repo_origin


def parse_args(args):
    parser = ArgumentParser(
        description="Create a new repo based on template"
        " repo.\nExample: create_from_template PCS1906/foo PCS1906/bar"
        " -sb source_branch -db destination_branch -d ."
    )
    parser.add_argument(
        "src_repo",
        help="The repo from which to create the new repo (must exist)"
        ", e.g. organisation/project/repo1",
    )
    parser.add_argument(
        "dest_repo",
        help="The new repo (must not exist), " "e.g. organisation/project/repo2",
    )
    parser.add_argument(
        "-sb",
        "--src_branch",
        default="develop",
        help="The branch to use, e.g. develop",
    )
    parser.add_argument(
        "-db",
        "--dest_branch",
        default="develop",
        help="The branch to create, e.g. develop. "
        "It will be set as default branch on created repo",
    )
    parser.add_argument(
        "-d",
        "--work_dir",
        default=".",
        help="The directory to create local repo in",
    )
    parser.add_argument(
        "-r",
        "--replacements",
        default=[],
        nargs="+",
        help=(
            "Pairs of replacements, e.i. the first in the pair is a string in "
            "the src_repo that will be replaced to the second in the pair in the dest_repo"
        ),
    )
    return parser.parse_args(args)


def main(
    src_repo,
    src_branch,
    dest_repo,
    dest_branch,
    work_dir,
    replacements,
):
    if len(replacements) % 2 > 0:
        print("Error: Replacements must be pairs")
        exit(1)

    clone_url, dest_repo_dir = setup(
        src_repo,
        src_branch,
        dest_repo,
        dest_branch,
        work_dir,
    )

    replacement_pairs = list(zip(replacements[0::2], replacements[1::2]))
    update(replacement_pairs, dest_repo_dir)

    commit_all(dest_repo_dir)

    dest_repo_origin = create_and_push_to_dest_repo(
        dest_branch,
        dest_repo,
        dest_repo_dir,
    )
    print("Created", dest_repo_origin)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import sys
from os import environ

from requests import delete, get, post, put


def get_page(uri, params, limit, start):
    query = f"&{'&'.join([f'{k}={v}' for k, v in params.items()])}" if params else ""
    response = call(f"{uri}?limit={limit}&start={start}{query}")
    return (
        response["size"],
        response["values"],
        response["isLastPage"],
        response.get("nextPageStart", -1),
    )


def get_all(uri, params=None, take=sys.maxsize):
    limit = min(take, 1000)
    start = 0
    is_last_page = False

    result = []
    while not is_last_page and len(result) < take:
        size, values, is_last_page, start = get_page(uri, params, limit, start)

        if size:
            result += values
    return result[:take]


def call(
    uri,
    request_data=None,
    method="GET",
    only_response_code=False,
    rest_url=environ.get("BITBUCKET_REST_URL", None),
    username=environ.get("BITBUCKET_USERNAME", None),
    password=environ.get("BITBUCKET_PASSWORD", None),
    verify_certificate=not environ.get("BITBUCKET_IGNORE_CERTIFICATE", None),
):
    """Makes a REST call to Bitbucket rest api 1.0.
    Depends on three environment variables:
    * BITBUCKET_REST_URL, e.g. http://myorg.com/stash
    * BITBUCKET_USERNAME, the bitbucket username
    * BITBUCKET_PASSWORD, the bitbucket password

    Args:
        uri (str): e.g. "/rest/api/1.0/projects/{project}/repos/{repo}/branches?filterText={branch}"
        request_data (dict): the JSON request
        method (str): http method
        only_response_code (bool): default False
        rest_url: default environ.get('BITBUCKET_REST_URL', None)
        username (str): default environ.get('BITBUCKET_USERNAME', None)
        password (str): default environ.get('BITBUCKET_PASSWORD', None)
        verify_certificate: True if https certificate should be verified

    Return:
          the JSON response
    """

    m = {
        "DELETE": delete,
        "GET": get,
        "POST": post,
        "PUT": put,
    }[method]
    # print(username, password, rest_url, request_data, uri)
    params = {
        "url": f"{rest_url}{uri}",
        "auth": (username, password),
        "verify": verify_certificate,
    }
    if request_data:
        params["json"] = request_data

    response = m(**params)
    return (
        response.status_code
        if only_response_code
        else response.json()
        if response.text
        else response.status_code()
    )



#!/usr/bin/env python3
import argparse
from os.path import normpath
from sys import argv

from vang.bitbucket.get_repos import get_all_repos


def get_clone_urls(keys, command=False, branch=False, flat=False):
    for repo in get_all_repos(keys):
        project, repo, clone_url = (
            repo["project"]["key"],
            repo["slug"],
            repo["links"]["clone"][0]["href"],
        )
        if command:
            clone_dir = normpath(
                f'{project}/{repo if flat else repo.replace(".", "/")}'
            )
            branch_spec = f"-b {branch} " if branch else ""
            clone_command = f"git clone {branch_spec}{clone_url} {clone_dir}"
            yield clone_dir, project, repo, clone_command
        else:
            yield None, project, repo, clone_url


def main(projects, command, branch, flat):
    for clone_dir, project, repo, clone_output in get_clone_urls(
        projects,
        command,
        branch,
        flat,
    ):
        print(clone_output)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get clone urls for projects from Bitbucket."
    )
    parser.add_argument("projects", nargs="+", help="The projects.")
    parser.add_argument(
        "-c", "--command", help="Print as clone commands", action="store_true"
    )
    parser.add_argument("-b", "--branch", help="Add -b <branch> to clone commands")
    parser.add_argument(
        "-f",
        "--flat",
        help="Make clone commands clone to flat structure",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



import re
from itertools import count


def _get_rc_number(version):
    m = re.search(r"rc([0-9]+)", version)
    return int(m[1]) if m else -1


def _get_version_number(version):
    version = version.replace("_", ".")
    try:
        numbers = [
            int(x) for x in re.match(r"([0-9]+)\.([0-9]+)\.([0-9]+)", version).groups()
        ]
        return numbers
    except AttributeError as e:
        raise ValueError(f"version must be semver {version}")


def _get_max(versions):
    if not versions:
        return None
    version_numbers = [_get_version_number(v) for v in versions]
    if len(version_numbers) == 1:
        return versions[0]
    max_version = versions[0]
    max_version_number = version_numbers[0]
    for i, v in zip(count(1), version_numbers[1:]):
        new_max = True
        if v[0] < max_version_number[0]:
            new_max = False
        if v[0] == max_version_number[0]:
            if v[1] < max_version_number[1]:
                new_max = False
            if v[1] == max_version_number[1]:
                if v[2] < max_version_number[2]:
                    new_max = False
        if new_max:
            max_version = versions[i]
            max_version_number = v
    if "rc" not in max_version:
        return max_version
    else:
        max_rc = max(
            _get_rc_number(v)
            for v in versions
            if _get_version_number(v) == max_version_number
        )
        return next(
            v
            for v in versions
            if _get_rc_number(v) == max_rc
            and _get_version_number(v) == max_version_number
        )


def get_latest_snapshot(versions):
    filtered_versions = [
        v for v in versions if "snapshot" in v.lower() and "rc" not in v.lower()
    ]
    return _get_max(filtered_versions)


def get_latest_rc(versions):
    filtered_version = [v for v in versions if "rc" in v.lower()]
    return _get_max(filtered_version)


def get_latest_fixed(versions):
    filtered_versions = [
        v for v in versions if "rc" not in v.lower() and "snapshot" not in v.lower()
    ]
    return _get_max(filtered_versions)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import environ
from os.path import basename
from pprint import pprint
from sys import argv
from typing import Dict

from requests import get

from vang.azdo.list_projects import list_projects

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = "https://dev.azure.com"


def get_project_id(
    token: str,
    organization: str,
    project: str,
    verify_certificate: bool,
    api_version: str = "6.1-preview.4",
) -> str:
    for p in list_projects(token, organization, verify_certificate, api_version)[
        "value"
    ]:
        if p["name"] == project:
            return p["id"]
    return "project_not_found"


def get_project(
    token: str,
    organization: str,
    project: str,
    verify_certificate: bool = True,
    api_version: str = "6.1-preview.4",
) -> Dict:
    project_id = get_project_id(
        token, organization, project, verify_certificate, api_version
    )
    url = f"{base_url}/{organization}/_apis/projects/{project_id}?api-version={api_version}"
    params = {"url": url, "auth": ("", token), "verify": verify_certificate}
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = get(**params)
    logger.info(f"response.status_code: {response.status_code}")
    logger.info(f"response.text: {response.text}")
    response.raise_for_status()
    return loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(description="Get project")
    parser.add_argument(
        "--token",
        default=environ.get("AZDO_TOKEN", ""),
        help="The Azure DevOps authorisation token",
    )
    parser.add_argument(
        "--organisation",
        default=environ.get("AZDO_ORGANISATION", ""),
        help="The Azure DevOps organisation",
    )
    parser.add_argument(
        "--project",
        default=environ.get("AZDO_PROJECT", ""),
        help="The Azure DevOps project",
    )
    parser.add_argument(
        "-au",
        "--azure_devops_url",
        default="https://dev.azure.com",
        help="The Azure DevOps REST API base url",
    )

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        "-i", "--project_id", action="store_true", help="Get only project id"
    )

    return parser.parse_args(args)


def main(
    token: str, organisation: str, project: str, azure_devops_url: str, project_id: bool
) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    project = get_project(token, organisation, project)
    if not project_id:
        pprint(project)
    else:
        print(project["id"])


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.jenkins.api import call


def trigger_builds(names):
    return [
        (
            name,
            call(
                f"/job/{name}/build",
                method="POST",
                only_response_code=True,
            ),
        )
        for name in names
    ]


def parse_args(args):
    parser = argparse.ArgumentParser(description="Trigger Jenkins jobs")
    parser.add_argument(
        "job_names",
        nargs="+",
        help="Jenkins job names",
    )
    return parser.parse_args(args)


def main(job_names):
    for job_name, response_code in trigger_builds(job_names):
        print(job_name, response_code)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.api import get_all


def get_projects():
    return get_all("/rest/api/1.0/projects")


def main(key=False):
    for project in get_projects():
        if key:
            print(project["key"])
        else:
            print(f'{project["key"]}: {project["name"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get projects from Bitbucket")
    parser.add_argument(
        "-k", "--key", help="Print only project key", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_tags import get_tags
from vang.bitbucket.utils import get_repo_specs


def has_tag(repo_specs, tag):
    for spec in repo_specs:
        tags = [t["displayId"] for spec, t in get_tags((spec,), tag)]
        yield spec, tag in tags


def main(tag, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, has in has_tag(specs, tag):
        print(f"{spec[0]}/{spec[1]}, {tag}: {has}")


def parse_args(args):
    parser = argparse.ArgumentParser(description="Check repository tags in Bitbucket")
    parser.add_argument("tag", help="The tag to check")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from glob import glob
from os.path import exists


def get_pom_path(pom_dir):
    pom_path = f"{pom_dir}/pom.xml"
    if exists(pom_path):
        return pom_path

    pom_paths = glob(f"{pom_dir}/*.pom", recursive=True)
    if not pom_paths:
        raise ValueError(f"No pom in {pom_dir}")
    if len(pom_paths) == 1:
        return pom_paths[0]
    return sorted(pom_paths, key=len)[0]  # The pom without timestamp


def get_artifact_base_uri(repository, group_id, artifact_id, version):
    return f'/{repository}/{"/".join(group_id.split("."))}/{artifact_id}/{version}'



#!/usr/bin/env python3
import argparse
import traceback
from json import load
from os import makedirs
from sys import argv

from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.has_branch import has_branch
from vang.core.core import is_included
from vang.pio.shell import run_commands


def should_be_cloned(project, repo, project_config, has_branch_map):
    return has_branch_map[(project, repo)] and is_included(
        repo,
        project_config["excludes"],
        project_config["includes"],
    )


def clone(commands, root_dir):
    makedirs(root_dir, exist_ok=True)
    yield from run_commands(
        [(cmd, root_dir) for cmd in commands], max_processes=25, timeout=60, check=False
    )


def get_projects_commands(projects, branch=None, flat=False):
    return (
        (clone_dir, project, repo, command)
        for clone_dir, project, repo, command in get_clone_urls(
            projects,
            True,
            branch,
            flat,
        )
    )


def get_repos_commands(repos, branch=None, flat=False):
    projects = {repo.split("/")[0] for repo in repos}

    commands = []
    clone_urls_map = {
        f"{project}/{repo}": (clone_dir, project, repo, command)
        for clone_dir, project, repo, command in get_clone_urls(
            projects,
            True,
            branch,
            flat,
        )
    }
    for repo_spec in repos:
        if repo_spec in clone_urls_map:
            commands.append(clone_urls_map[repo_spec])
        else:
            print(f"Warning! Non existing repo: {repo_spec}")

    return commands


def get_config_commands(config, branch=None, flat=False):
    clone_url_specs = tuple(
        get_clone_urls(
            config["projects"], True, branch if branch else config["branch"], flat
        )
    )
    has_branch_specs = has_branch(
        [(p, r) for d, p, r, c in clone_url_specs], config["branch"]
    )
    has_branch_map = {repo_spec: has for repo_spec, has in has_branch_specs}

    return (
        (clone_dir, project, repo, command)
        for clone_dir, project, repo, command in clone_url_specs
        if should_be_cloned(project, repo, config["projects"][project], has_branch_map)
    )


def main(root_dir, projects=None, repos=None, config=None, branch=None, flat=False):
    n = 1
    if projects:
        commands = get_projects_commands(projects, branch, flat)
    elif repos:
        commands = get_repos_commands(repos, branch, flat)
    else:
        with open(config, "rt", encoding="utf-8") as cfg:
            commands = get_config_commands(load(cfg), branch, flat)

    for process in clone(
        [command for clone_dir, project, repo, command in commands],
        root_dir,
    ):
        try:
            print(str(n).zfill(2), process.stdout.decode(), end="")
            n += 1
        except OSError:  # pragma: no cover
            print(traceback.format_exc())


def parse_args(args):
    parser = argparse.ArgumentParser(description="Clone Bitbucket repos")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-p", "--projects", nargs="+", help="Bitbucket projects, e.g key1 key2"
    )
    group.add_argument(
        "-r", "--repos", nargs="+", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument(
        "-c", "--config", help="Configuration file, see clone_repos.json"
    )
    parser.add_argument(
        "-b",
        "--branch",
        help="The clone branch. Overrides branch in configuration file (-c)",
    )
    parser.add_argument(
        "-d", "--root_dir", default=".", help="The directory to clone into"
    )
    parser.add_argument(
        "-f", "--flat", help="Clone to flat structure", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
""" Makes Maven project. """

from os import makedirs

from os.path import normpath

POM_TEMPLATE = """<project xmlns="http://maven.apache.org/POM/4.0.0"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
        http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>
    <groupId>###group_id###</groupId>
    <artifactId>###artifact_id###</artifactId>
    <version>###version###</version>
    <name>${project.groupId}:${project.artifactId}</name>
    <description>${project.artifactId}</description>
    <packaging>###packaging###</packaging>
    <properties>
        <maven.compiler.source>###java_version###</maven.compiler.source>
        <maven.compiler.target>###java_version###</maven.compiler.target>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
    </properties>
    <build>
        <plugins>
            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-compiler-plugin</artifactId>
                <version>3.8.1</version>
                <configuration>
                    <source>${maven.compiler.source}</source>
                    <target>${maven.compiler.target}</target>
                    <compilerId>groovy-eclipse-compiler</compilerId>
                </configuration>
                <dependencies>
                    <dependency>
                        <groupId>org.codehaus.groovy</groupId>
                        <artifactId>groovy-eclipse-compiler</artifactId>
                        <version>3.7.0</version>
                    </dependency>
                    <!-- for 2.8.0-01 and later you must have an explicit dependency on groovy-eclipse-batch -->
                    <dependency>
                        <groupId>org.codehaus.groovy</groupId>
                        <artifactId>groovy-eclipse-batch</artifactId>
                        <version>3.0.8-01</version>
                    </dependency>
                </dependencies>
            </plugin>
        </plugins>
    </build>
    <dependencies>
        <dependency>
            <groupId>org.codehaus.groovy</groupId>
            <artifactId>groovy</artifactId>
            <version>3.0.8</version>
        </dependency>
    </dependencies>   
</project>
"""


def get_pom(java_version, group_id, artifact_id, version, packaging):
    """Returns pom content."""
    return (
        POM_TEMPLATE.replace("###java_version###", java_version)
        .replace("###group_id###", group_id)
        .replace("###artifact_id###", artifact_id)
        .replace("###version###", version)
        .replace("###packaging###", packaging)
    )


def make_dirs(output_dir, group_id, artifact_id, packaging):
    """Make standard dirs of Maven project."""
    makedirs(output_dir)
    if packaging in ["jar", "war"]:
        package_path = normpath("/".join(group_id.split(".") + artifact_id.split(".")))
        for p in [
            "{}/src/main/java/{}",
            "{}/src/main/resources/{}",
            "{}/src/test/java/{}",
            "{}/src/test/resources/{}",
        ]:
            makedirs(normpath(p.format(output_dir, package_path)))
        if packaging in ["war"]:
            makedirs(normpath(f"{output_dir}/src/main/webapp"))


def make_project(output_dir, java_version, group_id, artifact_id, version, packaging):
    """Makes Maven project."""
    make_dirs(output_dir, group_id, artifact_id, packaging)
    pom = get_pom(java_version, group_id, artifact_id, version, packaging)
    with open(normpath(f"{output_dir}/pom.xml"), "wt", encoding="utf-8") as pom_file:
        pom_file.write(pom)


def main():
    group_id = str(input("groupId (default mygroup): ") or "mygroup")
    artifact_id = str(input("artifactId (default slask): ") or "slask")
    version = str(input("version (default 1.0.0-SNAPSHOT): ") or "1.0.0-SNAPSHOT")
    packaging = str(input("packaging (default jar): ") or "jar")
    java_version = str(input("javaVersion (default 11): ") or "11")
    output_dir = normpath("/".join(artifact_id.split(".")))
    make_project(output_dir, java_version, group_id, artifact_id, version, packaging)


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.definition_utils import get_definition, get_definition_name


def get_build_definition(template, organisation, project, repo, branch, comment=None):
    return get_definition(
        template,
        {
            "name": get_definition_name(project, repo, branch),
            "organisation": organisation,
            "project": project,
            "repo": repo,
            "branch": branch,
        },
        {"comment": comment} if comment else {},
    )


def create_build_definition(organisation, project, build_definition):
    return call(
        # f'/{organisation}/{project}/_apis/build/definitions?api-version=3.2',
        f"/{organisation}/{project}/_apis/build/definitions?api-version=3.2",
        request_data=build_definition,
        method="POST",
        only_response_code=True,
    )


def main(project, repo, branch, template, comment=None):
    organisation, project = project.split("/")
    response = create_build_definition(
        organisation,
        project,
        get_build_definition(template, organisation, project, repo, branch, comment),
    )
    print(response)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Create TFS build definitions")
    parser.add_argument("project", help="TFS projects, e.g organisation/project")
    parser.add_argument("repo", help="The TFS git repo name, e.g. spam.eggs")
    parser.add_argument("branch", help="The TFS git repo branch, e.g. 1.0.x")
    parser.add_argument(
        "template",
        help="The TFS build definition template file, e.g. build_definition_template.json",
    )
    parser.add_argument(
        "-c",
        "--comment",
        help="The comment to add to the build definition, e.g. a commit sha of the build definition template",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory import utils
from vang.artifactory import api
from vang.maven.pom import get_pom_info


def delete_maven_artifact(repository, pom_dirs):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(utils.get_pom_path(pom_dir))
        base_uri = utils.get_artifact_base_uri(
            repository,
            pom_info["group_id"],
            pom_info["artifact_id"],
            pom_info["version"],
        )
        yield api.call(base_uri, method="DELETE")


def main(repository, pom_dirs):
    for response in delete_maven_artifact(repository, pom_dirs):
        print(response)


def parse_args(args):
    parser = ArgumentParser(description="Delete maven artifact from Artifactory")
    parser.add_argument("repository", help="Artifactory repository, e.g. z-release")
    parser.add_argument(
        "-d",
        "--pom_dirs",
        nargs="*",
        default=["."],
        help="Maven pom directories to extract artifact information from",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from itertools import product
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import call
from vang.bitbucket.utils import get_repo_specs

hook_template = {
    "title": None,
    "url": None,
    "committersToIgnore": "",
    "branchesToIgnore": "",
    "enabled": True,
    "tagCreated": False,
    "branchDeleted": False,
    "branchCreated": False,
    "repoPush": True,
    "prDeclined": False,
    "prRescoped": False,
    "prMerged": False,
    "prReopened": False,
    "prUpdated": False,
    "prCreated": False,
    "prCommented": False,
    "buildStatus": False,
}


def get_uri(spec):
    return f"/rest/webhook/1.0/projects/{spec[0]}/repos/{spec[1]}/configurations"


def delete_web_hook(spec, hook_id):
    return call(f"{get_uri(spec)}/{hook_id}", only_response_code=True, method="DELETE")


def create_web_hook(spec, hook):
    return call(
        get_uri(spec), only_response_code=True, method="POST", request_data=hook
    )


def get_web_hooks(spec):
    return call(get_uri(spec))


def set_repo_web_hooks(spec, hooks):
    try:
        # Delete existing web hooks
        for hook in get_web_hooks(spec):
            delete_response_code = delete_web_hook(spec, hook["id"])
            if delete_response_code != 204:
                return (
                    spec,
                    False,
                    f'Can not delete {hook["id"]}: {delete_response_code}',
                )

        # Create web hooks
        for hook in hooks:
            create_response_code = create_web_hook(spec, hook)
            if create_response_code != 200:
                return spec, False, f"Can not create {hook}: {create_response_code}"

        return spec, True, f"Set webhooks {hooks}"
    except IOError as ioe:
        return spec, False, ioe


def set_web_hooks(specs, hooks, max_processes=10):
    with Pool(processes=max_processes) as pool:
        return pool.starmap(set_repo_web_hooks, product(specs, (hooks,)))


def main(title, url, dirs, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    hook = dict(hook_template, title=title, url=url)
    hooks = (hook,)
    for spec, result, message in set_web_hooks(specs, hooks):
        if result:
            print(f"spec[0]/{spec[1]}: {result}")
        else:
            print(f"{spec[0]}/{spec[1]}: {result} {message}")


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Set Bitbucket webhooks. "
        "The hooks are based on a template defined in this script."
    )
    parser.add_argument("title", help="The webhook title")
    parser.add_argument("url", help="The webhook url")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from.",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from datetime import datetime
from functools import reduce
from multiprocessing.dummy import Pool
from re import match
from typing import Callable, Iterable, Any, Dict, Mapping


def create_timestamp() -> str:
    return datetime.now().strftime("%Y%m%dT%H%M%S.%f")


def has_match(s: str, regexps: Iterable[str]) -> bool:
    return any((match(regexp, s) for regexp in regexps))


def is_included(
    name: str, excludes: Iterable[str] = None, includes: Iterable[str] = None
) -> bool:
    return not (excludes and has_match(name, excludes)) and (
        not includes or has_match(name, includes)
    )


def pmap(
    f: Callable[[Any], Any],
    iterable: Iterable[Any],
    chunksize: int = None,
    processes: int = 25,
) -> Iterable[Any]:
    with Pool(processes) as pool:
        return pool.map(f, iterable, chunksize)


def pmap_ordered(
    f: Callable[[Any], Any],
    iterable: Iterable[Any],
    chunksize: int = 1,
    processes: int = 10,
) -> Iterable[Any]:
    with Pool(processes=processes) as pool:
        completed_processes = pool.map(f, iterable, chunksize=chunksize)
        for cp in completed_processes:
            yield cp


def pmap_unordered(
    f: Callable[[Any], Any],
    iterable: Iterable[Any],
    chunksize: int = 1,
    processes: int = 10,
) -> Iterable[Any]:
    with Pool(processes=processes) as pool:
        completed_processes = pool.imap_unordered(f, iterable, chunksize=chunksize)
        for cp in completed_processes:
            yield cp


def select_keys(d: Mapping[Any, Any], keys: Iterable[Any]) -> Dict[Any, Any]:
    return {k: v for k, v in d.items() if k in keys}


def get_in(seq, keys):
    return reduce(lambda mem, k: mem[k], keys, seq) or None



#!/usr/bin/env python3

from argparse import ArgumentParser
from glob import glob
from os import environ
from os.path import basename
from sys import argv

from vang.maven.pom import get_pom_info
from vang.nexus3.upload import upload
from vang.nexus3.utils import get_artifact_base_uri, get_pom_path


def get_pom_publish_name(pom_path, artifact_id, version):
    pom_name = pom_path.split("/")[-1]
    return (
        pom_name if pom_name.split(".")[-1] == "pom" else f"{artifact_id}-{version}.pom"
    )


def get_publish_data(artifact_base_uri, path, name):
    return {"file_path": path, "repository_path": f"{artifact_base_uri}/{name}"}


def publish_maven_artifact(repository, pom_dirs, url, username, password):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(get_pom_path(pom_dir))
        base_uri = get_artifact_base_uri(
            pom_info["group_id"], pom_info["artifact_id"], pom_info["version"]
        )

        publish_data = [
            get_publish_data(
                base_uri,
                pom_info["pom_path"],
                get_pom_publish_name(
                    pom_info["pom_path"], pom_info["artifact_id"], pom_info["version"]
                ),
            )
        ] + [
            get_publish_data(base_uri, path, basename(path))
            for path in glob(f"{pom_dir}/**/*.jar", recursive=True)
            + glob(f"{pom_dir}/**/*.war", recursive=True)
        ]

        yield [
            upload(
                pd["file_path"],
                repository,
                pd["repository_path"],
                url,
                username,
                password,
            )
            for pd in publish_data
        ]


def main(repository, dirs, url, username, password):
    for response in publish_maven_artifact(repository, dirs, url, username, password):
        print(response)


def parse_args(args):
    parser = ArgumentParser(description="Publish maven artifact to Nexus3")
    parser.add_argument("repository", help="Nexus3 repository, e.g. z-release")
    parser.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Maven pom directories to extract artifact information from",
    )
    parser.add_argument(
        "-l",
        "--url",
        default=environ.get("NEXUS3_REST_URL", None),
        help="Nexus3 url, e.g. http://nexus_host:8080",
    )
    parser.add_argument(
        "-u",
        "--username",
        default=environ.get("NEXUS3_USERNAME", None),
        help="Nexus3 username",
    )
    parser.add_argument(
        "-p",
        "--password",
        default=environ.get("NEXUS3_PASSWORD", None),
        help="Nexus3 password",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from itertools import product
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import call
from vang.bitbucket.utils import get_repo_specs


def enable_repo_web_hook(spec, url):
    uri = (
        f"/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}"
        + "/settings/hooks/com.atlassian.stash.plugin.stash-web-post-"
        + "receive-hooks-plugin:postReceiveHook/enabled"
    )
    request_data = {"hook-url-0": url}
    return spec, call(uri, request_data, "PUT")


def enable_web_hook(repo_specs, url, max_processes=10):
    with Pool(processes=max_processes) as pool:
        return pool.starmap(enable_repo_web_hook, product(repo_specs, [url]))


def main(url, dirs, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, response in enable_web_hook(specs, url):
        print(
            f"{spec[0]}/{spec[1]}: "
            f'{"enabled" if response["enabled"] else " not enabled"}'
        )


def parse_args(args):
    parser = argparse.ArgumentParser(description="Enables Bitbucket webhooks.")
    parser.add_argument(
        "url",
        help="The url which to send repo info to, e.g. "
        "http://10.20.30.40:8002/wildcat/webhook/",
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from.",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from base64 import encodebytes
from json import loads
from os import environ
from urllib.request import Request, urlopen


def call(
    uri,
    extra_headers=None,
    request_data=None,
    method="GET",
    rest_url=environ.get("ARTIFACTORY_REST_URL", None),
    username=environ.get("ARTIFACTORY_USERNAME", None),
    password=environ.get("ARTIFACTORY_PASSWORD", None),
):
    """Makes a REST call to Artifactory.

    Args:
        uri (str): e.g. "/{repository/{'/'.join(group_id.split('.'))}/{artifact_id}/{version}""
        extra_headers: default None
        request_data (bytes): the request payload
        method (str): http method
        rest_url: artifactory rest url
        username: artifactory username
        password: artifactory password

    Return:
          the JSON response (dict)
    """
    auth = f"{username}:{password}"
    basic_auth_header = f'Basic {encodebytes(auth.encode()).decode("UTF-8").strip()}'
    url = f"{rest_url}{uri}"
    headers = {"Authorization": basic_auth_header, "Content-Type": "application/json"}
    if extra_headers:
        headers.update(extra_headers)

    request = create_request(headers, method, request_data, url)
    response = urlopen(request)
    response_data = response.read()
    return loads(response_data.decode("UTF-8")) if response_data else response.getcode()


def create_request(headers, method, request_data, url):
    return Request(url, request_data if request_data else None, headers, method=method)



#!/usr/bin/env python3

import logging
import sys
import traceback
from subprocess import PIPE, STDOUT, run
from typing import Tuple, Union, Iterable, Any

from vang.core.core import pmap_unordered

log = logging.getLogger(__name__)


def run_command(
    command: str,
    return_output: bool = False,
    cwd: str = None,
    check: bool = True,
    timeout: int = None,
) -> Tuple[int, Union[str, int]]:
    cp = run(
        command,
        cwd=cwd,
        stdout=PIPE,
        stderr=STDOUT,
        check=check,
        timeout=timeout,
        shell=True,
    )
    return (
        (cp.returncode, cp.stdout.decode().strip()) if return_output else cp.returncode
    )


def run_commands(
    commands_and_cwds: Iterable[Iterable[str]],
    max_processes: int = 10,
    check: bool = True,
    timeout: int = None,
) -> Iterable[Any]:
    """
    Runs commands in parallel.

    :param commands_and_cwds: Pairs of command and working directory for the command
    :param max_processes: The max number of parallel processes
    :param check: Check for Exception
    :param timeout: The timeout of each process
    :return: CompletedProcess (yielded)
    """

    def f(cc):
        cmd, cwd = cc
        try:
            return run(
                cmd,
                cwd=cwd,
                stdout=PIPE,
                stderr=STDOUT,
                check=check,
                timeout=timeout,
                shell=True,
            )
        except Exception as e:
            traceback.print_exc(file=sys.stdout)
            raise e

    yield from pmap_unordered(
        f, commands_and_cwds, processes=max_processes, chunksize=1
    )



#!/usr/bin/env python3
import argparse
from sys import argv
from typing import Union
from zipfile import ZipFile

from requests import get


def download(
    output_file: Union[str, bytes, int], url: str, username: str, password: str
) -> None:
    with open(output_file, "wb") as f:
        f.write(get(url, auth=(username, password)).content)


def extract_zip(zip_file):
    with ZipFile(zip_file, "r") as z:
        z.extractall()


def main(
    output_file: Union[str, bytes, int],
    url: str,
    username: str,
    password: str,
    extract: bool,
) -> None:
    download(output_file, url, username, password)
    if extract:
        extract_zip(output_file)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Download file (requires: pipenv install requests)"
    )
    parser.add_argument("output_file", help="output file, e.g. ./eggs.zip")
    parser.add_argument(
        "url",
        help="url to download, e.g. http://localhost:9000/job/CSL_xml.ws_1.0.x/3/execution/node/3/ws/target/surefire-reports/*zip*/surefire-reports.zip",
    )
    parser.add_argument("-u", "--username", help="username for basic auth")
    parser.add_argument("-p", "--password", help="password for basic auth")
    parser.add_argument(
        "-e",
        "--extract",
        help="if download zip, use this option to extract",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":
    main(**parse_args(argv[1:]).__dict__)



import os

from setuptools import setup


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="vang",
    version="1.3.1",
    author="Magnus Bjuvensjö",
    author_email="bjuvensjo@gmail.com",
    description=("Scripts"),
    license="MIT",
    keywords="python script artifactory azdo bitbucket git github jenkins maven nexus3 tfs",
    url="https://github.com/bjuvensjo/scripts",
    packages=[
        "vang.artifactory",
        "vang.azdo",
        "vang.bitbucket",
        "vang.core",
        "vang.git",
        "vang.github",
        "vang.jenkins",
        "vang.maven",
        "vang.misc",
        "vang.nexus3",
        "vang.pio",
        "vang.tfs",
        "vang.wildcat",
    ],
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=["more-itertools", "requests", "pytest"],
)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory import api


def list_repos(repo_type=None, only_key=False, only_url=False):
    result = api.call("/api/repositories")
    if repo_type:
        result = [r for r in result if r["type"] == repo_type]
    if only_key:
        result = [r["key"] for r in result]
    if only_url:
        result = [r["url"] for r in result]
    return result


def main(repo_type, only_key, only_url):
    for repo in list_repos(repo_type, only_key, only_url):
        print(repo)


def parse_args(args):
    parser = ArgumentParser(description="List repositories in Artifactory")
    parser.add_argument(
        "-t",
        "--repo_type",
        default=None,
        help="Repository type, e.g. LOCAL, REMOTE, VIRTUAL. Default is all.",
    )
    parser.add_argument(
        "-k", "--only_key", help="Print only repo key", action="store_true"
    )
    parser.add_argument(
        "-u", "--only_url", help="Print only repo url", action="store_true"
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
from glob import glob
from itertools import chain

from os.path import realpath
from sys import argv


def get_reactor_summary(mvn_log):
    """Returns reactor summary."""
    start = False
    lines = []
    single_project_result = "FAILURE"
    single_project = ""

    for line in mvn_log:
        # Multi module project build
        if re.match(r".*Reactor Summary.*", line):
            start = True
        if start:
            lines.append(line.strip())
        if start and re.match(r"^.INFO. -.*", line):
            break

        # Single project build
        if re.match(r".*Building.*", line) and not single_project:
            single_project = line.strip().replace(" Building", "")
        if single_project and re.match(r".*BUILD SUCCESS.*", line):
            single_project_result = "SUCCESS"
            break

    if lines:
        return lines[2:-1]

    return [f"{single_project} {single_project_result}"]


def get_project(line):
    """Returns project of reactor line."""
    start = re.search(r" ", line).end()
    end = re.search(r" ", line[start:]).start() + start
    return line[start:end]


def get_successes(reactor_summary):
    """Returns success projects"""
    return [get_project(l) for l in reactor_summary if re.search("SUCCESS", l)]


def get_failures(reactor_summary):
    """Returns failure projects"""
    return [get_project(l) for l in reactor_summary if not re.search("SUCCESS", l)]


def read_lines(log_file):  # pragma: no cover
    with open(log_file, "rt", encoding="utf-8") as f:
        return f.readlines()


def get_summary(mvn_log_files, do_print=False):
    summaries = tuple(
        chain.from_iterable(
            [get_reactor_summary(read_lines(log_file)) for log_file in mvn_log_files]
        )
    )
    result = get_successes(summaries), get_failures(summaries)
    if do_print:
        print_summary(*result)
    return result


def print_summary(successes, failures):
    length = 80
    lines = []

    def banner(text="", fill=" "):
        s = " " + text + " " if text else text
        l = int(length / 2 + len(s) / 2)
        return s.rjust(l, fill).ljust(length, fill)

    def artifact(the_id, success, fill="."):
        outcome = "SUCCESS" if success else "FAILURE"
        return the_id + outcome.rjust(length - len(the_id), fill)

    for a_text in ("", "Summary", ""):
        lines.append(banner(a_text, "*"))
    for an_id in sorted(successes):
        lines.append(artifact(an_id, True))
    for an_id in sorted(failures):
        lines.append(artifact(an_id, False))
    for a_text in ("",):
        lines.append(banner(a_text, "*"))

    print("\n".join(lines))


def main(roots, log_file_name):
    successes, failures = get_summary(
        [
            realpath(p)
            for root in roots
            for p in glob(f"{root}/**/{log_file_name}", recursive=True)
        ]
    )
    print_summary(successes, failures)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Prints reactor summary of Maven build log files"
    )
    parser.add_argument(
        "-d",
        "--roots",
        help="Root directories in which to find Maven log files",
        nargs="*",
        default=["."],
    )
    parser.add_argument(
        "-l",
        "--log_file_name",
        help="Name of Maven build log file(s)",
        default="mvn.log",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from pprint import pprint
from re import fullmatch
from sys import argv

from requests import get

from vang.core.core import pmap_ordered


def get_repo_content(repo_key, url, username, password):
    response = get(
        f"{url}/api/storage/{repo_key}?list&deep=1&listFolders=1",
        auth=(username, password),
    )
    if response.status_code == 200:
        return response.json()
    else:
        response.raise_for_status()


def filter_repo_files(repo_files, excludes):
    return [
        f
        for f in repo_files
        if not (f["folder"] or any(fullmatch(e, f["uri"]) for e in excludes))
    ]


def create_key(artifact_spec):
    return (
        artifact_spec["uri"]
        if artifact_spec["folder"]
        else f'{artifact_spec["uri"]}#{artifact_spec["sha1"]}'
    )


def map_to_map(repo_files):
    return {create_key(s): s for s in repo_files}


def diff(
    a_repo_files, b_repo_files, excludes=(".*maven-metadata.xml",), only_keys=True
):
    a_repo_files_map, b_repo_files_map = list(
        pmap_ordered(
            lambda repo_files: map_to_map(filter_repo_files(repo_files, excludes)),
            (a_repo_files, b_repo_files),
            processes=2,
        )
    )

    a_key_set = set(a_repo_files_map.keys())
    b_key_set = set(b_repo_files_map.keys())
    a_only_keys = a_key_set.difference(b_key_set)
    b_only_keys = b_key_set.difference(a_key_set)

    if only_keys:
        return a_only_keys, b_only_keys
    else:
        return (v for k, v in a_repo_files_map.items() if k in a_only_keys), (
            v for k, v in b_repo_files_map.items() if k in b_only_keys
        )


def diff_repos(repo_specs, excludes=(".*maven-metadata.xml",), only_keys=True):
    a_repo_content, b_repo_content = list(
        pmap_ordered(lambda spec: get_repo_content(**spec), repo_specs, processes=2)
    )
    return diff(a_repo_content["files"], b_repo_content["files"], excludes, only_keys)


def main(
    a_repo_key,
    a_artifactory_url,
    a_artifactory_username,
    a_artifactory_password,
    b_repo_key,
    b_artifactory_url,
    b_artifactory_username,
    b_artifactory_password,
):
    specs = (
        {
            "repo_key": a_repo_key,
            "artifactory_url": a_artifactory_url,
            "username": a_artifactory_username,
            "password": a_artifactory_password,
        },
        {
            "repo_key": b_repo_key,
            "artifactory_url": b_artifactory_url,
            "username": b_artifactory_username,
            "password": b_artifactory_password,
        },
    )
    a_only_keys, b_only_keys = diff_repos(specs)

    if a_only_keys:
        print("### Only in a ###")
        pprint(a_only_keys)
    if b_only_keys:
        print("### Only in b ###")
        pprint(b_only_keys)
    print("Done!")


def parse_args(args):
    parser = ArgumentParser(description="Diff content of repositories")
    parser.add_argument("a_repo_key", help="a repo key")
    parser.add_argument("a_artifactory_url", help="a artifactory url")
    parser.add_argument("a_artifactory_username", help="a artifactory username")
    parser.add_argument("a_artifactory_password", help="a artifactory password")
    parser.add_argument("b_repo_key", help="b repo key")
    parser.add_argument("b_artifactory_url", help="b artifactory url")
    parser.add_argument("b_artifactory_username", help="b artifactory username")
    parser.add_argument("b_artifactory_password", help="b artifactory password")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
import shlex
from os.path import abspath
from subprocess import run, CalledProcessError
from sys import argv

from vang.pio.shell import run_command


def get_origin_remote_url(repo_dir, remote):
    command = "git remote -v"
    try:
        rc, output = run_command(command, True, repo_dir)
        for line in output.split("\n"):
            if remote in line:
                return re.split(r"[\t ]+", line)[1]
    except CalledProcessError as cpe:
        print(cpe)
    return None


def open_remote(repo_dir, remote):
    url = get_origin_remote_url(repo_dir, remote)
    if url:
        print(f"Opening {url}")
        run(shlex.split(f"open {url}"))
    else:
        print(f"Found no url to open in {repo_dir}")


def main(repo_dir, remote):  # pragma: no cover
    open_remote(abspath(repo_dir), remote)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Open remote url in browser")
    parser.add_argument(
        "-d",
        "--repo_dir",
        default=".",
        help="Git directory to extract repo information from",
    )
    parser.add_argument("-r", "--remote", default="origin", help="The remote to open")

    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import shlex
from os import environ
from os.path import abspath
from subprocess import run
from sys import argv

from vang.bitbucket.utils import get_branch, get_project_and_repo


def open_remote(
    git_dir,
    repo,
    project,
    base_url,
):
    if project:
        url = f"{base_url}/projects/{project}"
    elif repo:
        p, r = repo.split("/")
        url = f"{base_url}/projects/{p}/repos/{r}/browse"
    else:
        p, r = get_project_and_repo(git_dir)
        url = "".join(
            [
                f"{base_url}/projects/{p}/repos/{r}/commits?",
                f"until=refs%2Fheads%2F{get_branch(git_dir)}&merges=include",
            ]
        )
    run(shlex.split(f"open {url}"))


def main(repo_dir, repo, project, baseurl):
    open_remote(abspath(f"{repo_dir}/.git"), repo, project, baseurl)


def parse_args(args):
    parser = argparse.ArgumentParser(description="Open Bitbucket url in browser")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--repo_dir",
        default=".",
        help="Git directory to extract repo information from",
    )
    group.add_argument("-r", "--repo", help="Repo, e.g. key1/repo1")
    group.add_argument("-p", "--project", help="Project, e.g. key1")
    parser.add_argument(
        "-b", "--baseurl", default=environ.get("BITBUCKET_REST_URL", None), help=""
    )

    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import hashlib
from itertools import count
from os import walk
from os.path import exists
from re import match
from sys import argv


def sha256sum(filename):
    h = hashlib.sha256()
    b = bytearray(128 * 1024)
    mv = memoryview(b)
    with open(filename, "rb", buffering=0) as f:
        for n in iter(lambda: f.readinto(mv), 0):
            h.update(mv[:n])
    return h.hexdigest()


def diff(master_dir, compare_dir, ignore):
    """
    Returns files in master dir that are not in compare_dir,
    and file in master dir that differ (sha256) from files in compare_dir

    :param master_dir:
    :param compare_dir:
    :param ignore:
    :return:
    """
    missing_files = []
    diffing_files = []

    for root, dirs, files in walk(master_dir):
        for f in files:
            if not any([match(p, f) for p in ignore]):
                mf = f"{root}/{f}"
                cf = f"{compare_dir}/{root[len(master_dir) + 1:]}/{f}"

                if exists(cf):
                    if sha256sum(mf) != sha256sum(cf):
                        diffing_files.append((mf, cf))
                else:
                    missing_files.append(mf)

    return missing_files, diffing_files


def main(master_dir, compare_dir, ignore, only_files=False):
    mfs, dfs = diff(master_dir, compare_dir, ignore)

    if only_files:
        for mf in mfs:
            print(mf)
        for mf, df in dfs:
            print(mf)
    else:
        print("##### Missing files #####")
        for n, mf in zip(count(), mfs):
            print(n, mf)

        print("##### Diffing files #####")
        for n, df in zip(count(), dfs):
            print(n, df)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Returns files in master dir that are not "
        "in compare_dir, and file in master dir that differ "
        "(sha256) from files in compare_dir"
    )
    parser.add_argument("master_dir", help="master dir")
    parser.add_argument("compare_dir", help="compare dir")
    parser.add_argument(
        "-i",
        "--ignore",
        nargs="*",
        default=(
            r".*lastUpdated",
            r"_remote.repositories",
            r".*sha1",
            r"maven-metadata-local.xml",
            r"maven-metadata-local.xml",
        ),
    )
    parser.add_argument("-f", "--only_files", action="store_true")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from pprint import pprint
from re import match

from vang.pio.shell import run_command
from vang.pio.synchronize_dirs import synchronize_dirs


def apply_patch(patch_repo, apply_repo, ref):
    print("Apply patch", ref, patch_repo, apply_repo, ref)
    rc, output = run_command(f"git checkout {ref}", True, patch_repo)
    print(output)

    synchronize_dirs(patch_repo, apply_repo)
    rc, output = run_command("git status", True, apply_repo)
    print(output)

    if "nothing to commit" in output:
        rc, output = run_command(f"git tag -a {ref} -m {ref}", True, apply_repo)
        print(output)
    else:
        for cmd in [
            "git add --all",
            f"git commit -m {ref}",
            f"git tag -a {ref} -m {ref}",
        ]:
            rc, output = run_command(cmd, True, apply_repo)
            print(output)

    return ref


def get_refs(repo, ref_pattern):
    return [
        ref
        for ref in run_command("git tag", True, repo)[1].split("\n")
        if match(r"{}".format(ref_pattern), ref)
    ]


def get_unpatched_refs(patchs_refs, applied_refs):
    return [p for p in patchs_refs if p not in applied_refs]


def is_valid(patchs_refs, applied_refs):
    return all([p == a for p, a in zip(patchs_refs, applied_refs)]) and not len(
        applied_refs
    ) > len(patchs_refs)


def main(patch_repo, ref_pattern, apply_repo):
    patchs_refs = get_refs(patch_repo, ref_pattern)
    applied_refs = get_refs(apply_repo, ref_pattern)
    if is_valid(patchs_refs, applied_refs):
        unpatched_refs = get_unpatched_refs(patchs_refs, applied_refs)
        applied_patches = []
        for ref in unpatched_refs:
            apply_patch(patch_repo, apply_repo, ref)
            applied_patches.append(ref)
        return applied_patches
    else:
        raise ValueError("Tags are not valid.")


def parse_args(args):
    parser = ArgumentParser(
        description="Create patches of refs and applies, "
        + "commits and refs them in another repo."
    )
    parser.add_argument("ref_pattern", help="A ref pattern.")
    parser.add_argument("apply_repo", help="The repo to apply patches to.")
    parser.add_argument(
        "-p",
        "--patch_repo",
        help="The repo to patch from.",
        default=".",
    )
    parser.add_argument(
        "-o",
        "--output",
        help="A directory to put patches in.",
        default="./patch",
    )
    return parser.parse_args(args)


# rm -rf sign/patch; rm -rf lf-sign; md lf-sign; cd lf-sign; git init
# main(
#     '/Users/ei4577/slask/slask/PCS1806/sign',
#     'release.*',
#     '/Users/ei4577/slask/slask/PCS1806/lf-sign',
# )
# rm -rf lf-process.mortgage; md lf-process.mortgage; cd lf-process.mortgage; git init
the_applied_patches = main(
    "/Users/ei4577/slask/slask/PCS1806/process.mortgage",
    "release.*",
    "/Users/ei4577/slask/slask/PCS1806/lf-process.mortgage",
)
pprint(the_applied_patches)

# if __name__ == '__main__':  # pragma: no cover
#     args = parse_args(argv[1:])
#     main(args.patch_repo, args.ref_pattern, args.output, args.apply_repo)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.api import get_all
from vang.bitbucket.utils import get_repo_specs
from vang.core.core import pmap_unordered


def get_branches(repo_specs, branch="", max_processes=10):
    return pmap_unordered(
        lambda s: (
            s,
            get_all(
                f"/rest/api/1.0/projects/{s[0]}/repos/{s[1]}/branches",
                {"filterText": branch},
            ),
        ),
        repo_specs,
        processes=max_processes,
    )


def main(branch="", name=False, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, branches in get_branches(specs, branch):
        for b in branches:
            if name:
                print(b["displayId"])
            else:
                print(f'{spec[0]}/{spec[1]}: {b["displayId"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Get repository branches from Bitbucket"
    )
    parser.add_argument("-b", "--branch", help="Branch filter", default="")
    parser.add_argument(
        "-n", "--name", help="Print only branch name", action="store_true"
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Git directories to extract repo information from",
    )
    group.add_argument(
        "-r", "--repos", nargs="*", help="Repos, e.g. key1/repo1 key2/repo2"
    )
    group.add_argument("-p", "--projects", nargs="*", help="Projects, e.g. key1 key2")
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import hashlib
import json
from argparse import ArgumentParser
from glob import glob
from sys import argv

from vang.artifactory import api
from vang.artifactory.utils import get_artifact_base_uri, get_pom_path
from vang.maven.pom import get_pom_info


def read_file(file_path):  # pragma: no cover
    with open(file_path, "rb") as f:
        return f.read()


def get_checksums(the_bytes):
    return (
        hashlib.md5(the_bytes).hexdigest(),
        hashlib.sha1(the_bytes).hexdigest(),
        hashlib.sha256(the_bytes).hexdigest(),
    )


def get_checksum_headers(md5, sha1, sha256):
    return {
        "X-Checksum-Md5": md5,
        "X-Checksum-Sha1": sha1,
        # sha256 Not yet supported...
        "X-Checksum-Sha256": sha256,
    }


def get_pom_publish_name(pom_path, artifact_id, version):
    pom_name = pom_path.split("/")[-1]
    return (
        pom_name if pom_name.split(".")[-1] == "pom" else f"{artifact_id}-{version}.pom"
    )


def get_publish_data(artifact_base_uri, path, name):
    content = read_file(path)
    md5, sha1, sha256 = get_checksums(content)
    return {
        "content": content,
        "checksum_headers": get_checksum_headers(md5, sha1, sha256),
        "uri": f"{artifact_base_uri}/{name}",
    }


def publish_maven_artifact(repository, pom_dirs):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(get_pom_path(pom_dir))
        base_uri = get_artifact_base_uri(
            repository,
            pom_info["group_id"],
            pom_info["artifact_id"],
            pom_info["version"],
        )

        publish_data = [
            get_publish_data(
                base_uri,
                pom_info["pom_path"],
                get_pom_publish_name(
                    pom_info["pom_path"], pom_info["artifact_id"], pom_info["version"]
                ),
            )
        ] + [
            get_publish_data(base_uri, path, path.split("/")[-1])
            for path in glob(f"{pom_dir}/**/*.jar", recursive=True)
            + glob(f"{pom_dir}/**/*.war", recursive=True)
        ]

        yield [
            api.call(pd["uri"], pd["checksum_headers"], pd["content"], "PUT")
            for pd in publish_data
        ]


def main(repository, dirs):
    for response in publish_maven_artifact(repository, dirs):
        print(json.dumps(json.loads(str(response).replace("'", '"')), indent=2))


def parse_args(args):
    parser = ArgumentParser(description="Publish maven artifact to Artifactory")
    parser.add_argument("repository", help="Artifactory repository, e.g. z-release")
    parser.add_argument(
        "-d",
        "--dirs",
        nargs="*",
        default=["."],
        help="Maven pom directories to extract artifact information from",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory.api import call


def get_repo_content(repo_key):
    return call(f"/api/storage/{repo_key}?list&deep=1&listFolders=1")


def list_repo_content(repo_key, only_files=True):
    result = get_repo_content(repo_key)
    result = result["files"]
    if only_files:
        result = [f["uri"] for f in result if not f["folder"]]
    return result


def main(repo_key, only_files):
    for f in list_repo_content(repo_key, only_files):
        print(f)


def parse_args(args):
    parser = ArgumentParser(description="List repository content")
    parser.add_argument("repo_key", help="Repository key, e.g. libs-release")
    parser.add_argument(
        "-f",
        "--only_files",
        help="Print only ordinary files, i.e. not directories",
        action="store_true",
    )
    return parser.parse_args(args)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call


def get_projects(organisations, project_specs=False, names=False):
    if not organisations:
        return []
    projects = [
        (o, p)
        for o in organisations
        for p in call(f"/{o}/_apis/projects?api-version=3.2")["value"]
    ]
    if names:
        return [project[1]["name"] for project in projects]
    if project_specs:
        return [f'{project[0]}/{project[1]["name"]}' for project in projects]

    return projects


def parse_args(args):
    parser = argparse.ArgumentParser(description="Get TFS projects")
    parser.add_argument(
        "organisations", nargs="+", help="TFS organisations, e.g organisation"
    )
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "-n", "--names", action="store_true", help="Get only project names"
    )
    group.add_argument(
        "-p",
        "--project_specs",
        action="store_true",
        help="Get only organisation/project",
    )

    return parser.parse_args(args)


def main(organisations, project_specs, names):
    for a_project in get_projects(organisations, project_specs, names):
        print(a_project)


if __name__ == "__main__":  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)
